/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. Icf not, see <http://www.gnu.org/licenses/>.
 */
var isAdmin = people.isAdmin(person);
var Evaluator =
{
   /**
    * Cache for cm:person objects
    */
   PeopleObjectCache: {},

   /**
    * Gets / caches a person object
    *
    * @method getPersonObject
    * @param nodeRef {string} NodeRef of a cm:person object
    */
   getPersonObject: function Evaluator_getPersonObject(nodeRef)
   {
      if (nodeRef == null || nodeRef == "")
      {
         return null;
      }

      if (typeof Evaluator.PeopleObjectCache[nodeRef] == "undefined")
      {
         var person = search.findNode(nodeRef);
         Evaluator.PeopleObjectCache[nodeRef] =
         {
            userName: person.properties.userName,
            firstName: person.properties.firstName,
            lastName: person.properties.lastName,
            displayName: (person.properties.firstName + " " + person.properties.lastName).replace(/^\s+|\s+$/g, "")
         };
         if (person.assocs["cm:avatar"] != null)
         {
            Evaluator.PeopleObjectCache[nodeRef].avatar = person.assocs["cm:avatar"][0];
         }
      }
      return Evaluator.PeopleObjectCache[nodeRef];
   },

    /**
     * Cache for cm:authorityContainer objects
     */
    AuthorityContainerObjectCache: {},

    /**
     * Gets / caches an authorityContainer object
     *
     * @method getAuthorityContainerObject
     * @param nodeRef {string} NodeRef of a cm:authority object
     */
    getAuthorityContainerObject: function Evaluator_getAuthorityContainerObject(nodeRef)
    {
        if (nodeRef == null || nodeRef == "")
        {
            return null;
        }

        if (typeof Evaluator.AuthorityContainerObjectCache[nodeRef] == "undefined")
        {
            var authority = search.findNode(nodeRef);
            Evaluator.AuthorityContainerObjectCache[nodeRef] =
            {
                authorityName: authority.properties.authorityName,
                displayName: authority.properties.authorityDisplayName.replace(/^\s+|\s+$/g, "")
            };
        }
        return Evaluator.AuthorityContainerObjectCache[nodeRef];
    },
    
    hasUploadRedlinedPermission: function Evaluator_hasUploadRedlinedPermission(node)
    {
      var shortName = node.getSiteShortName();
      var civilSiteShortName = siteHelper.getRelatedCivilSiteShortName(shortName);
	  if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList")
	      && node.hasAspect("lyse:engineeringDocumentAspect")
	      && siteHelper.isContractorUser(civilSiteShortName)
		  && node.properties["lysedl:workflowId"] == null
		  && node.properties["lyse:cdlReviewStatus"] != "void"
		  && (node.properties["lyse:cdlReviewStatus"] == "ifc" || node.properties["lyse:cdlReviewStatus"] == "redlined")){
		  return true;
	  }else{
		  return false;
	  }
   },  

   /**
    * Cache for nodes that are subtypes of cm:cmobject
    */
   ContentObjectCache: {},

   /**
    * Gets / caches a content object
    *
    * @method getContentObject
    * @param nodeRef {string} NodeRef
    */
   getContentObject: function Evaluator_getContentObject(nodeRef)
   {
      if (nodeRef == null || nodeRef == "")
      {
         return null;
      }

      if (typeof Evaluator.ContentObjectCache[nodeRef] == "undefined")
      {
         var node = search.findNode(nodeRef);
         try
         {
            Evaluator.ContentObjectCache[nodeRef] = node;
         }
         catch(e)
         {
            // Possibly a stale indexed node
            return null;
         }
      }
      return Evaluator.ContentObjectCache[nodeRef];
   },
   

   /**
    * @return {boolean}
    */
   canShowWorkflowHistory: function Evaluator_canShowWorkflowHistory(node)
   {
      return (node.properties["lysedl:historicalWorkflowIds"] != null);
   },

   /**
    * @return {boolean}
    */
   canShowShareHistory: function Evaluator_canShowShareHistory(node)
   {
      return (node.properties["lyse:shareHistoryWorkflowIds"] != null);
   },



   /**
    * Checks if a node can be edited by the current user.
    *
    * @param node
    * @constructor
    */
   canEdit: function Evaluator_canEdit(node)
   {
	   return lyseWorkflowService.canEdit(node);
   },

   
   /**
    * Generate displayValue and any extra metadata for this field
    *
    * @method decorateFieldData
    * @param objData {object} Object literal containing this field's data
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} false to prevent this field being added to the output stream.
    */
   decorateFieldData: function Evaluator_decorateFieldData(objData, node)
   {
      var value = objData.value,
         type = objData.type,
         obj;

      if (type == "cm:authority")
      {
         // cm:authority is the super type of both cm:person and cm:authorityContainer, now find the correct type and use that
         var authority = search.findNode(value);
         if (logger.isLoggingEnabled()) {
           logger.log("Authority: " + authority)
         }
         type = authority.typeShort;
         if (logger.isLoggingEnabled()) {
            logger.log("Authority type: " + type)
         }
      }

      if (type == "cm:person")
      {
         obj = Evaluator.getPersonObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.displayName;
         objData.metadata = obj.userName;
      }
      else if (type == "cm:authorityContainer")
      {
         obj = Evaluator.getAuthorityContainerObject(value);
         if (obj == null)
         {
             return false;
         }
         objData.displayValue = obj.displayName;
         objData.metadata = obj.authorityName;
      }
      else if (type == "category")
      {
    	 obj = Evaluator.getContentObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.name;
      }

      else if (type == "cm:folder")
      {
         obj = Evaluator.getContentObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.displayPath.substring(companyhome.name.length() + 1);
         objData.metadata = "container";
      }
      else if (type.indexOf(":") > 0 && node.isSubType("cm:cmobject"))
      {
         obj = Evaluator.getContentObject(value);
         if (obj == null || !obj.hasPermission("Read"))
         {
            return false;
         }
         objData.displayValue = obj.properties["cm:name"];
         objData.theType = type;
         objData.metadata = obj.isContainer ? "container" : "document";
      }
      return true;
   },
   
   /**
    * Translates a List fieldDefinition
    *
    * @method translateField
    * @param objDef {FieldDefinition} objDef
    * @param value {String} default value
    */
   translateField: function Evaluator_translateField(objDef, value)
   {
      if (objDef == null || objDef == "")
      {
         return null;
      }
      if (objDef.constraints != null) 
      {
         for ( var i=0, len= objDef.constraints.size(); i<len; ++i )
         {          
             if ("LIST" == objDef.constraints.get(i).type) 
             {
                var allowedV = objDef.constraints.get(i).parameters.allowedValues;
                for (var j=0; j<allowedV.size(); ++j )
                {   
                    var allowedVasString = "" + allowedV.get(j);
                    var allValSplit = allowedVasString.split("|");
                    if (value == allValSplit[0]) {
                       return allValSplit[1];
                    }
                }
              }
          }
      }
      return value;
   },
   
   /**
    * Node Evaluator - main entrypoint
    */
   run: function Evaluator_run(node, fields)
   {
      var permissions = {},
         actionSet = "",
         actionLabels = {},
         createdBy = Common.getPerson(node.properties["cm:creator"]),
         modifiedBy = Common.getPerson(node.properties["cm:modifier"]),
         nodeData = {};

      /**
       * PERMISSIONS
       */
      logger.log("BEFORE PERMISSIONS CHECK");
      try {
      permissions =
      {
         "admin": isAdmin,
         "engRedlined": Evaluator.hasUploadRedlinedPermission(node)
      };

      }
      catch (ex) {
         logger.log(ex);
         throw ex;
      }

      // Use the form service to parse the required properties
      scriptObj = formService.getForm("node", node.nodeRef, fields, fields);

      // Make sure we can quickly look-up the Field Definition within the formData loop...
      var objDefinitions = {};
      for each (formDef in scriptObj.fieldDefinitions)
      {
         objDefinitions[formDef.dataKeyName] = formDef;
      }

      // Populate the data model
      var formData = scriptObj.formData.data;
      for (var k in formData)
      {
         var isAssoc = k.indexOf("assoc") == 0,
            value = formData[k].value,
            values,
            type = isAssoc ? objDefinitions[k].endpointType : objDefinitions[k].dataType,
            endpointMany = isAssoc ? objDefinitions[k].endpointMany : false,
            objData =
            {
               type: type
            };

         logger.log("Type: " + type + ", endpointMany: " + endpointMany);
         if (value instanceof java.util.Date)
         {
            objData.value = utils.toISO8601(value);
            objData.displayValue = objData.value;
            nodeData[k] = objData;
         }
         else if (endpointMany)
         {
            if (value.length() > 0)
            {
               values = value.split(",");
               nodeData[k] = [];
               for each (value in values)
               {
                  var objLoop =
                  {
                     type: objData.type,
                     value: value,
                     displayValue: value
                  };

                  if (Evaluator.decorateFieldData(objLoop, node))
                  {
                     nodeData[k].push(objLoop);
                  }
               }
            }
         }
         else
         {
            if (logger.isLoggingEnabled()) {
               logger.log("k: " + k)
            }
            if (k == "prop_cm_creator") {
               // This is a text field, resolve the person so we can display the full name instead of only the username.
               // The decorateFieldData will take care of the rest, together with extended-datagrid.js..
               var person = people.getPerson(value);
               objData.value = person.nodeRef.toString();
               objData.type = person.typeShort;
            }
            else {
               objData.value = value;
               objData.displayValue = isAssoc ? value : Evaluator.translateField(objDefinitions[k],value);
            }

            if (Evaluator.decorateFieldData(objData, node))
            {
               nodeData[k] = objData;
            }
         }
      }

      return(
      {
         node: node,
         nodeData: nodeData,
         actionSet: actionSet,
         actionPermissions: permissions,
         createdBy: createdBy,
         modifiedBy: modifiedBy,
         tags: node.tags,
         actionLabels: actionLabels,
         isLocked: node.isLocked,
         workflowTasks: lyseWorkflowService.getAssignedTasksForDatalist(node, true),
         responsibles: lyseWorkflowService.getResponsiblesForDatalistItem(node)
      });
   }
};
