/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. Icf not, see <http://www.gnu.org/licenses/>.
 */

var isAdmin = people.isAdmin(person);
var currentUserId = person.properties.userName;
var ctx = Packages.org.springframework. web.context.ContextLoader.getCurrentWebApplicationContext();
var asTextConfig = ctx.getBean('lyse.responsibleGroupsPolicy').getTypes();

var Evaluator =
{
   /**
    * Cache for cm:person objects
    */
   PeopleObjectCache: {},

   /**
    * Gets / caches a person object
    *
    * @method getPersonObject
    * @param nodeRef {string} NodeRef of a cm:person object
    */
   getPersonObject: function Evaluator_getPersonObject(nodeRef)
   {
      if (nodeRef == null || nodeRef == "")
      {
         return null;
      }

      if (typeof Evaluator.PeopleObjectCache[nodeRef] == "undefined")
      {
         var person = search.findNode(nodeRef);
         Evaluator.PeopleObjectCache[nodeRef] =
         {
            userName: person.properties.userName,
            firstName: person.properties.firstName,
            lastName: person.properties.lastName,
            displayName: (person.properties.firstName + " " + person.properties.lastName).replace(/^\s+|\s+$/g, "")
         };
         if (person.assocs["cm:avatar"] != null)
         {
            Evaluator.PeopleObjectCache[nodeRef].avatar = person.assocs["cm:avatar"][0];
         }
      }
      return Evaluator.PeopleObjectCache[nodeRef];
   },

    /**
     * Cache for cm:authorityContainer objects
     */
    AuthorityContainerObjectCache: {},

    /**
     * Gets / caches an authorityContainer object
     *
     * @method getAuthorityContainerObject
     * @param nodeRef {string} NodeRef of a cm:authority object
     */
    getAuthorityContainerObject: function Evaluator_getAuthorityContainerObject(nodeRef)
    {
        if (nodeRef == null || nodeRef == "")
        {
            return null;
        }

        if (typeof Evaluator.AuthorityContainerObjectCache[nodeRef] == "undefined")
        {
            var authority = search.findNode(nodeRef);
            Evaluator.AuthorityContainerObjectCache[nodeRef] =
            {
                authorityName: authority.properties.authorityName,
                displayName: authority.properties.authorityDisplayName.replace(/^\s+|\s+$/g, "")
            };
        }
        return Evaluator.AuthorityContainerObjectCache[nodeRef];
    },

   /**
    * Cache for nodes that are subtypes of cm:cmobject
    */
   ContentObjectCache: {},

   /**
    * Gets / caches a content object
    *
    * @method getContentObject
    * @param nodeRef {string} NodeRef
    */
   getContentObject: function Evaluator_getContentObject(nodeRef)
   {
      if (nodeRef == null || nodeRef == "")
      {
         return null;
      }

      if (typeof Evaluator.ContentObjectCache[nodeRef] == "undefined")
      {
         var node = search.findNode(nodeRef);
         try
         {
            Evaluator.ContentObjectCache[nodeRef] = node;
         }
         catch(e)
         {
            // Possibly a stale indexed node
            return null;
         }
      }
      return Evaluator.ContentObjectCache[nodeRef];
   },

   /**
    * Only allow deletion of draft data list items.
    *
    * @method hasDeletePermission
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} false to prevent this field being added to the output stream.
    */
   hasDeletePermission: function Evaluator_getDeletePermission(node)
   {
      if (Evaluator.los.isLos(node)){
        // Everybody is collaborator.
        // See: ProjectPermissionPolicy.setInterfacePermissions
        //
        // Only owner and admin should be able to delete
        // (checks might be redundant, but does no harm.)
        // It should not be possible to delete when closed

        if (Evaluator.los.hasStatusClosed(node)) {
          return false;
        }
        else {
          if (node.owner==currentUserId){
            return node.hasPermission("Delete");
          }
          if (isAdmin){
            return node.hasPermission("Delete");
          }
          return false;
        }
      }

      if (node.type == "{http://ns.lyse.no/model/datalist/1.0}document"
          || node.type == "{http://ns.lyse.no/model/datalist/1.0}hseMeetingList"
          || node.type == "{http://ns.lyse.no/model/datalist/1.0}hseNec"
          || node.type == "{http://ns.lyse.no/model/datalist/1.0}melList"
		  || node.type == "{http://ns.lyse.no/model/datalist/1.0}tenderList"
          || node.type == "{http://ns.lyse.no/model/datalist/1.0}variousDocumentList"
          || node.type == "{http://ns.lyse.no/model/datalist/1.0}mccVariousDocumentList"
          || node.type == "{http://ns.lyse.no/model/datalist/1.0}grunnEierMatriseDataList"
          || node.type == "{http://ns.lyse.no/model/datalist/1.0}civilVariousDocumentList"
          || node.type == "{http://ns.lyse.no/model/datalist/1.0}civilQualityControl")
      {
         return node.hasPermission("Delete");
      // #33604, never allow a user to delete a meeting
      }else  if (node.type == "{http://ns.lyse.no/model/datalist/1.0}meetingMinutes"){
         return false;
      }else{
         return node.hasPermission("Delete")
         && Evaluator.isDraft(node)
         && node.properties["lysedl:workflowId"] == null
         && node.properties["lysedl:historicalWorkflowIds"] == null;
      }

   },
   /**
    * If this is a contractors document list record, there should be
    * an uploadNewRevision-action. We solve this by using a new permission
    * type. No active workflow allowed.
    *
    * @method hasUploadNewRevisionPermission
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a cdlList (ie show the upload new revision action).
    */
   hasUploadNewRevisionPermission: function Evaluator_hasUploadNewRevisionPermission(node)
   {
	  if (((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList" ||node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList" )
		  && node.hasPermission("Write")
		  && node.properties["lysedl:workflowId"] == null
		  && node.properties["lyse:cdlReviewStatus"] != "void") || Evaluator.hasUploadNewEngRevisionPermission(node)){
		  return true;
	  }else if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}civilCdlDataList"||node.type == "{http://ns.lyse.no/model/datalist/1.0}civilContractDataList")
            && node.hasPermission("Write")
            && node.properties["lysedl:workflowId"] == null
            && node.properties["lyse:civilCdlReviewStatus"] != "void"){
            	  return true;
	  }else if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}companysDocumentDataList")
          && node.hasPermission("Write")
          && node.properties["lysedl:workflowId"] == null
          && node.properties["lyse:companysDocumentReviewStatus"] != "void"
          && siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyTech")){
          return true;
      }else{
		  return false;
	  }
   },


   hasUploadNewGenericDocumentPermission: function Evaluator_hasUploadNewGenericDocumentPermission(node)
   {
	  if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}genericDocumentList")
		  && node.hasPermission("Write")){
		  return true;
	  }else{
		  return false;
	  }
   },

   hasUploadNewEngRevisionPermission: function Evaluator_hasUploadNewEngRevisionPermission(node)
   {
	  if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList")
	      && node.hasAspect("lyse:engineeringDocumentAspect")
		  && node.hasPermission("Write")
		  && node.properties["lysedl:workflowId"] == null
		  && node.properties["lyse:engReviewStatus"] != "void"){
		  return true;
	  }else{
		  return false;
	  }
   },

   hasUploadAsBuiltPermission: function Evaluator_hasUploadAsBuiltPermission(node)
   {
	  if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList")
	      && node.hasAspect("lyse:engineeringDocumentAspect")
		  && node.hasPermission("Write")
		  && node.properties["lysedl:workflowId"] == null
		  && node.properties["lyse:engReviewStatus"] != "void"
		  && (node.properties["lyse:engReviewStatus"] == "ifc" || node.properties["lyse:engReviewStatus"] == "redlined" || node.properties["lyse:engReviewStatus"] == "asBuilt")){
		  return true;
	  }else{
		  return false;
	  }
   },

   hasUploadNewIFCDrawingPermission: function Evaluator_hasUploadNewEngRevisionPermission(node)
   {
	  if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList")
	      && node.hasAspect("lyse:engineeringDocumentAspect")
		  && node.hasPermission("Write")
		  && siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteEngineering")
		  && node.properties["lysedl:workflowId"] == null
		  && node.assocs["lyse:engIfcAttachment"] != null
		  && node.properties["lyse:engReviewStatus"] != "void"){
		  return true;
	  }else{
		  return false;
	  }
   },

   /**
    * If this is a contractors document list record which is not in an ongoing review
    * workflow, there should be an submitForReview-action. We solve this by using a new permission
    * type.
    *
    * @method okForReview
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a cdlList, civilCdlList, contractList, engList and
    *                   fulfills the requirements for starting a workflow.
    */
   okForReview: function Evaluator_okForReview(node)
   {
       var shortName = node.getSiteShortName();

       if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList"
           || node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList")
           && Evaluator.hasUploadNewRevisionPermission(node)
           && siteHelper.isContractorUser(shortName)
           && node.hasAspect("lyse:documentIsReviewed") == false
           && node.properties["lyse:cdlRevisionDate"] != null ){
           return true;
       }else if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}civilCdlDataList"
    	   || node.type == "{http://ns.lyse.no/model/datalist/1.0}civilContractDataList")
           && node.hasPermission("Write")
           && siteHelper.isContractorUser(shortName)
           && node.properties["lysedl:workflowId"] == null
           && node.hasAspect("lyse:documentIsReviewed") == false
           && node.properties["lyse:civilCdlReviewStatus"] != "void"){
           return true;
       }else if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}companysDocumentDataList")
           && node.hasPermission("Write")
           && siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyTech")
           && node.properties["lysedl:workflowId"] == null
           && node.hasAspect("lyse:documentIsReviewed") == false
           && node.properties["lyse:cdlRevisionDate"] != null
           && node.properties["lyse:companysDocumentReviewStatus"] != "void"){
           return true;
       }else{
           return false;
       }
   },

   okForAction: function Evaluator_okForAction(node)
   {
	  return (node.type == "{http://ns.lyse.no/model/datalist/1.0}issueList"
           && node.hasPermission("Write")
           && node.properties["lysedl:issueStatus"] == "draft"
           && node.properties["lysedl:workflowId"] == null);
   },

   okForInterfaceAction: function Evaluator_okForInterfaceAction(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}interfaceAction"
      && node.hasPermission("Write")
      && node.properties["lysedl:interfaceActionStatus"] == "draft"
      && node.properties["lysedl:workflowId"] == null);
   },

   /**
    * @return {boolean}
    */
   canShowWorkflowHistory: function Evaluator_canShowWorkflowHistory(node)
   {
      return (node.properties["lysedl:historicalWorkflowIds"] != null);
   },

   /**
    * @return {boolean}
    */
   canShowShareHistory: function Evaluator_canShowShareHistory(node)
   {
      return (node.properties["lyse:shareHistoryWorkflowIds"] != null);
   },

   /**
    * Check if a node has a draft status.
    *
    * @param node
    * @return {boolean}
    */
   isDraft: function Evaluator_isDraft(node)
   {
       return (node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList" && node.properties["lyse:cdlReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList" && node.properties["lyse:cdlReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}variationOrderRequestItem" && (node.properties["lysedl:vorStatus"] == "vor_draft" || node.properties["lysedl:vorStatus"] == "vo_draft"))
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}issueList" && node.properties["lysedl:issueStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}interfaceAction" && node.properties["lysedl:interfaceActionStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}ncrList" && node.properties["lysedl:ncrStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}irlDataList" && node.properties["lyse:irlReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}iasDataList" && node.properties["lyse:iasReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}epbDataList" && node.properties["lyse:epbReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}punchList" && node.properties["lyse:punchReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}issuePunchList" && node.properties["lyse:issuepunchReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilPunchList" && node.properties["lyse:civilpunchReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}qaList" && node.properties["lyse:qaReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}hseAction" && node.properties["lyse:hseActionReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}mccAction" && node.properties["lyse:mccActionReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}safeWorkPermitItem" && node.properties["lyse:swpStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}actionWithProject" && node.properties["lyse:actionWithProjectReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}siteQuery" && node.properties["lyse:siteQueryStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}correspondenceItem" && node.properties["lysedl:correspondenceStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}contact") // in interface site, special handling.
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}sjaDataList"  && node.properties["lyse:sjaStatus"] == "new")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}riskReducingMeasures" && node.properties["lyse:rrmStatus"] == "identified_risk")
       	   || (node.type == "{http://ns.lyse.no/model/datalist/1.0}ruiList" && node.properties["lyse:ruiReviewStatus"] == "new")
       	   || (node.type == "{http://ns.lyse.no/model/datalist/1.0}mccPackages" && node.properties["lyse:mccPackagesStatus"] == "new")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilMeasuredQuantity" && (node.properties["lyse:civilMqStatus"] == "not_started"))
       	   || (node.type == "{http://ns.lyse.no/model/datalist/1.0}mccCommissioningPackages" && node.properties["lyse:mccCommissioningPackagesStatus"] == "start")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilCdlDataList" && node.properties["lyse:civilCdlReviewStatus"] == "draft")
       	   || (node.type == "{http://ns.lyse.no/model/datalist/1.0}mccPackages" && node.properties["lyse:mccPackagesStatus"] == "new")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}milestoneItem" && node.properties["lysedl:milestoneStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilMilestone" && node.properties["lyse:civilMilestoneStatus"] == "draft")
       	   || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilTechnicalQuery" && node.properties["lyse:civilTqStatus"] == "draft")
       	   || (node.type == "{http://ns.lyse.no/model/datalist/1.0}siteQueries" && node.properties["lyse:siteQueryNgStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}companysDocumentDataList" && node.properties["lyse:companysDocumentReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}userFeedback" && node.properties["lysedl:userFeedbackStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilContractDataList" && node.properties["lyse:civilCdlReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}accommodationDataList" && node.properties["lyse:accoStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilCorrespondenceItem" && node.properties["lysedl:civilCorrespondenceStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilAction" && node.properties["lyse:civilActionReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilVariationOrderRequestItem" && (node.properties["lysedl:civilVorStatus"] == "vor_draft" || node.properties["lysedl:civilVorStatus"] == "vo_draft"));
   },

   /**
    * Checks if a node can be edited by the current user.
    *
    * @param node
    * @constructor
    */
   canEdit: function Evaluator_canEdit(node)
   {
    // Make sure noone deletes node after it has been closed.
    // Not even the owner.
    if (Evaluator.los.isLos(node)){
      if (Evaluator.los.hasStatusClosed(node)) {
        return false;
      }
    } else if(node.type == "{http://ns.lyse.no/model/datalist/1.0}riskReducingMeasures") {
      return (lyseWorkflowService.canEdit(node) && lyseWorkflowService.canStartRiskReducingMeasuresWorkflow(node));
    }

    return lyseWorkflowService.canEdit(node);
   },
   isMCCCommissioningPackageReady: function Evaluator_isMCCCommissioningPackageReady(node)
   {
    return lyseWorkflowService.isMCCCommissioningPackageReady(node);
   },
   canGenerateMeetingPdf: function Evaluator_canGenerateMeetingPdf(node)
   {
	   return node.type == "{http://ns.lyse.no/model/datalist/1.0}hseMeetingList";
   },
   canStartNewMQVersion: function Evaluator_startNewMQVersion(node)
   {
     return node.properties["lyse:civilMqStatus"] == "approved" &&
       ((siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteContractorConstructionManagement") || siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyConstructionManagement"))
       || (siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteContractorRep") || siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyRep")));  
   },
   canStartRiskReducingMeasuresWorkflow: function Evaluator_canStartRiskReducingMeasuresWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}riskReducingMeasures" && Evaluator.isDraft(node) && lyseWorkflowService.canStartRiskReducingMeasuresWorkflow(node));
   },

   canStartVariationOrderWorkflow: function Evaluator_canStartVariationOrderWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}variationOrderRequestItem" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartCivilVariationOrderWorkflow: function Evaluator_canStartCivilVariationOrderWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilVariationOrderRequestItem" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },

   canStartMilestoneWorkflow: function Evaluator_canStartMilestoneWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}milestoneItem" && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartCivilMilestoneWorkflow: function Evaluator_canStartCivilMilestoneWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilMilestone" && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartNcrWorkflow: function Evaluator_canStartNcrWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}ncrList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartIrlWorkflow: function Evaluator_canStartIrlWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}irlDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartIasWorkflow: function Evaluator_canStartIasWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}iasDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartEpbWorkflow: function Evaluator_canStartEpbWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}epbDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartPunchlistWorkflow: function Evaluator_canStartPunchlistWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}punchList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartIssuepunchlistWorkflow: function Evaluator_canStartIssuepunchlistWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}issuePunchList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartCivilPunchlistWorkflow: function Evaluator_canStartCivilPunchlistWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilPunchList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartQAListWorkflow: function Evaluator_canStartQAListWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}qaList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartCorrespondenceWorkflow: function Evaluator_canStartCorrespondenceWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}correspondenceItem" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartHseActionWorkflow: function Evaluator_canStartHseActionWorkflow(node)
   {
	   return (node.type == "{http://ns.lyse.no/model/datalist/1.0}hseAction" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartMccActionWorkflow: function Evaluator_canStartMccActionWorkflow(node)
   {
	   return (node.type == "{http://ns.lyse.no/model/datalist/1.0}mccAction" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartActionWithProjectWorkflow: function Evaluator_canStartActionWithProjectWorkflow(node)
   {
     return (node.type == "{http://ns.lyse.no/model/datalist/1.0}actionWithProject" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartSiteQueryWorkflow: function Evaluator_canStartSiteQueryWorkflow(node)
   {
       return (node.type == "{http://ns.lyse.no/model/datalist/1.0}siteQuery" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartRiskWorkflow: function Evaluator_canStartRiskWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}riskDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null);
   },
   canStartHSEWorkflow: function Evaluator_canStartHSEWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}sjaDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartRuiWorkflow: function Evaluator_canStartRuiWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}ruiList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node)
         && siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteHSEPrincipalEnterprise"));
   },
   canStartMCCPackagesWorkflow: function Evaluator_canStartMCCPackagesWorkflow(node)
   {
     return (node.type == "{http://ns.lyse.no/model/datalist/1.0}mccPackages" && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node) &&
         siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), search.findNode(node.assocs["lyse:mccPackagesContract"][0].nodeRef).properties["cm:authorityName"]));
   },
   canStartCivilMeasuredQuantityWorkflow: function Evaluator_canStartCivilMeasuredQuantityWorkflow(node)
   {
     return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilMeasuredQuantity" && node.properties["lysedl:workflowId"] == null &&  Evaluator.isDraft(node) &&  
            (siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteContractorRep") || siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyRep")));
   },
   canStartMCCCommissioningPackagesWorkflow: function Evaluator_canStartMCCCommissioningPackagesWorkflow(node)
   {
     return (node.type == "{http://ns.lyse.no/model/datalist/1.0}mccCommissioningPackages" && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node) &&
         siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteMCCCompanyMC"));
   },
   canStartCivilTechnicalQueryWorkflow: function Evaluator_canStartCivilTechnicalQueryWorkflow(node)
   {
      // Only initiator can start the workflow
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilTechnicalQuery" && node.hasPermission("Write") &&
        node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node)
      && ((siteHelper.isAssociated(person, node, "lyse:civilTqInitiatorUsers") ||
          siteHelper.isAssociated(person, node, "lyse:civilTqInitiatorGroups"))
        || node.owner == currentUserId));
   },
    canStartSiteQueryNgWorkflow: function Evaluator_canStartSiteQueryNgWorkflow(node)
    {
        // Only initiator can start the workflow
        return (node.type == "{http://ns.lyse.no/model/datalist/1.0}siteQueries" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node)
        && (siteHelper.isAssociated(person, node, "lyse:siteQueryNgInitiatorUsers") || siteHelper.isAssociated(person, node, "lyse:siteQueryNgInitiatorGroups")));
    },
   canStartCivilActionWorkflow: function Evaluator_canStartCivilActionWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilAction" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },

   canStartUnosysFeedbackWorkflow: function Evaluator_canStartUnosysFeedbackWorkflow(node)
   {
     return (node.type == "{http://ns.lyse.no/model/datalist/1.0}userFeedback" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartInterfaceAccommodationWorkflow: function Evaluator_canStartInterfaceAccommodationWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}accommodationDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canUpdateOrderAccommodationTask: function Evaluator_canUpdateOrderAccommodationTask(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}accommodationDataList"
            && node.properties["lyse:accoStatus"] == "close_order"
            && node.properties["lysedl:workflowId"] != null )
            && siteHelper.isContractorUser(node.getSiteShortName());
   },
   canStartCivilCorrespondenceWorkflow: function Evaluator_canStartCivilCorrespondenceWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilCorrespondenceItem" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },

   canVoidMilestone: function Evaluator_canVoidMilestone(node)
   {
       return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilMilestone" && siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyRep"));
   },
   canShowEditors: function Evaluator_canShowEditors(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}genericDocumentList");
   },

   canStartMCCSafeWorkPermitWorkflow: function Evaluator_canStartMCCSafeWorkPermitWorkflow(node)
   {
       return (node.type == "{http://ns.lyse.no/model/datalist/1.0}safeWorkPermitItem" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canCompleteCivilMeasuredQuantity: function Evaluator_canCompleteCivilMeasuredQuantity(node)
   {
     return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilMeasuredQuantity"
       && node.properties["lysedl:workflowId"] != null
       && node.properties["lyse:civilMqStatus"] == "not_completed"
         && (siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyRep")
             || siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyConstructionManagement")));
   },
   canIncompleteCivilMeasuredQuantity: function Evaluator_canIncompleteCivilMeasuredQuantity(node)
   {
     return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilMeasuredQuantity"
       && node.properties["lysedl:workflowId"] == null
       && node.properties["lyse:civilMqStatus"] == "completed"
       && (siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyRep")
           || siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyConstructionManagement")));
   },
   canCivilMilestoneSubmitInvoice: function Evaluator_canCivilMilestoneSubmitInvoice(node)
   {
     return (node.type == "{http://ns.lyse.no/model/datalist/1.0}civilMilestone"
       && node.properties["lysedl:workflowId"] == null
       && node.properties["lyse:civilMilestoneStatus"] == "draft"
       && node.properties["lyse:civilMilestoneInvoiceable"] == true
       && siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteCompanyRep"));
   },
    canRevokeMCCSafeWorkPermit: function Evaluator_canRevokeMCCSafeWorkPermit(node)
    {
        return siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteMCCSafetyManagers") && node.properties["lysedl:workflowId"] != null && node.properties["lyse:swpStatus"] == "approved_current";
    },

    canCompleteMCCSafeWorkPermit: function Evaluator_canCompleteMCCSafeWorkPermit(node)
    {
        return (siteHelper.hasSiteRole(siteHelper.getSiteShortName(node), "SiteMCCSafetyManagers") && node.properties["lysedl:workflowId"] != null &&
            (node.properties["lyse:swpStatus"] == "approved_expired" || node.properties["lyse:swpStatus"] == "rejected"))
    },

   /**
    * If this is the contractors document list there should be an multiple void-action.
    * We solve this by using a new permission type.
    *
    * @method canVoidCdlDocument
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a cdlList and its not in an existing workflow.
    */
   canVoidCdlDocument: function Evaluator_canVoidCdlDocument(node)
   {

       if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList"
           || node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList")
           && node.properties["lyse:cdlReviewStatus"] != "void"
           && node.hasPermission("Write")
           && node.properties["lysedl:workflowId"] == null){
           return true;
       }else if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}civilCdlDataList"
    	   || node.type == "{http://ns.lyse.no/model/datalist/1.0}civilContractDataList")
           && node.properties["lyse:civilCdlReviewStatus"] != "void"
           && node.hasPermission("Write")
           && node.properties["lysedl:workflowId"] == null){
           return true;
       }else if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}companysDocumentDataList")
           && node.properties["lyse:companysDocumentReviewStatus"] != "void"
           && node.hasPermission("Write")
           && node.properties["lysedl:workflowId"] == null){
           return true;
       }else{
           return false;
       }
   },

   /**
    * If this is the engineering's document list there should be an multiple revoke-action.
    * We solve this by using a new permission type.
    *
    * @method canRevokeEngDocument
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a cdlDataList and its in an existing workflow. It also has to have an IFC attacment and status have to be ifc.
    */
   canRevokeEngDocument: function Evaluator_canRevokeEngDocument(node)
   {
	  return ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList")
	      && node.hasAspect("lyse:engineeringDocumentAspect")
		  && node.assocs["lyse:engIfcAttachment"] != null
          && node.hasPermission("Write")
          && node.properties["lysedl:workflowId"] != null);
   },

   /**
    * If this is the hse nec list the attachments are handled in a document library view.
    * This method returns true if an icon for navigation to this lib should be present in the
    * datagrid.
    *
    * @method canHandleAttachments
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a hceNec list item.
    */
   canHandleAttachments: function Evaluator_canHandleAttachments(node)
   {
     // #36198 - For now this feature is hidden by request.
	   //return node.type == "{http://ns.lyse.no/model/datalist/1.0}hseNec";
     return false;
   },

   /**
    * Only true for project sites, issueLists, irlLists, embLists, vo/vor:s and company users
    *
    * @method createMeetingMinutes
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is the correct list, and the user has the authority
    */
   createMeetingMinutes: function Evaluator_createMeetingMinutes(node)
   {
	  if (node.type == "{http://ns.lyse.no/model/datalist/1.0}issueList" ||
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}iasDataList" ||
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}irlDataList" ||
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}interfaceAction" ||
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}epbDataList" ||
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}variationOrderRequestItem"||
              node.type == "{http://ns.lyse.no/model/datalist/1.0}civilVariationOrderRequestItem"){
		  var shortName = node.getSiteShortName();
		  var sitePreset = siteService.getSite(shortName).sitePreset;
		  if (sitePreset == "lyse-project" || sitePreset == "contractor-project"){
             return (siteHelper.isCompanyUser(shortName) || siteHelper.isContractorUser(shortName));
		  }else if (sitePreset == "interface-site"){
			  return true;
		  }
	  }
	  return false;
   },

   canStartShareWorkflow: function Evaluator_canStartShareWorkflow(node) {
      return node.hasPermission("Read");
   },


   /**
    * Generate displayValue and any extra metadata for this field
    *
    * @method decorateFieldData
    * @param objData {object} Object literal containing this field's data
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} false to prevent this field being added to the output stream.
    */
   decorateFieldData: function Evaluator_decorateFieldData(objData, node)
   {
      var value = objData.value,
         type = objData.type,
         obj;

      if (type == "cm:authority")
      {
         // cm:authority is the super type of both cm:person and cm:authorityContainer, now find the correct type and use that
         var authority = search.findNode(value);
         if (logger.isLoggingEnabled()) {
           //logger.log("Authority: " + authority)
         }
         type = authority.typeShort;
         if (logger.isLoggingEnabled()) {
            //logger.log("Authority type: " + type)
         }
      }

      if (type == "cm:person")
      {
         obj = Evaluator.getPersonObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.displayName;
         objData.metadata = obj.userName;
      }
      else if (type == "cm:authorityContainer")
      {
         obj = Evaluator.getAuthorityContainerObject(value);
         if (obj == null)
         {
             return false;
         }
         objData.displayValue = obj.displayName;
         objData.metadata = obj.authorityName;
      }
      else if (type == "category")
      {
    	 obj = Evaluator.getContentObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.name;
      }

      else if (type == "cm:folder")
      {
         obj = Evaluator.getContentObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.displayPath.substring(companyhome.name.length() + 1);
         objData.metadata = "container";
      }
      else if (type.indexOf(":") > 0 && node.isSubType("cm:cmobject"))
      {
         obj = Evaluator.getContentObject(value);
         if (obj == null || !obj.hasPermission("Read"))
         {
            return false;
         }
         objData.displayValue = obj.properties["cm:name"];
         objData.theType = type;
         objData.metadata = obj.isContainer ? "container" : "document";
      }
      return true;
   },

   /**
    * Translates a List fieldDefinition
    *
    * @method translateField
    * @param objDef {FieldDefinition} objDef
    * @param value {String} default value
    */
   translateField: function Evaluator_translateField(objDef, value)
   {
      if (objDef == null || objDef == "")
      {
         return null;
      }
      if (objDef.constraints != null)
      {
         for ( var i=0, len= objDef.constraints.size(); i<len; ++i )
         {
             if ("LIST" == objDef.constraints.get(i).type)
             {
                var allowedV = objDef.constraints.get(i).parameters.allowedValues;
                for (var j=0; j<allowedV.size(); ++j )
                {
                    var allowedVasString = "" + allowedV.get(j);
                    var allValSplit = allowedVasString.split("|");
                    if (value == allValSplit[0]) {
                       return allValSplit[1];
                    }
                }
              }
          }
      }
      return value;
   },

   canDeleteWorkflow:function(node)
   {
      return isAdmin && node.properties["lysedl:workflowId"] !== null;
   },

   permissions:function(node){
     if(node.isLocked) {
        return {
           "startShareForCommentingWorkflow": Evaluator.canStartShareWorkflow(node),
           "startShareForInformationWorkflow": Evaluator.canStartShareWorkflow(node),
           "unlockItem":scriptHelper.canUnlock(node),
           "cdlSubmitForReview":false,
           "showWorkflowHistory": Evaluator.canShowWorkflowHistory(node),
           "showShareHistory": Evaluator.canShowShareHistory(node),
           "incompleteCivilMeasuredQuantity" : Evaluator.canIncompleteCivilMeasuredQuantity(node),
        };
      }

      return {
         "admin": isAdmin,
         "create": node.hasPermission("CreateChildren"),
         "edit": Evaluator.canEdit(node),
         "isMCCCommissioningPackageReady": Evaluator.isMCCCommissioningPackageReady(node),
         "handleAttachments": Evaluator.canHandleAttachments(node),
         "delete": Evaluator.hasDeletePermission(node),
         "cdlRevise": Evaluator.hasUploadNewRevisionPermission(node),
         "genericDocumentUpload": Evaluator.hasUploadNewGenericDocumentPermission(node),
         "engAsBuilt": Evaluator.hasUploadAsBuiltPermission(node),
         "cdlSubmitForReview": Evaluator.okForReview(node),
         "cdlVoidDocument" :Evaluator.canVoidCdlDocument(node),
         "voidMilestone" :Evaluator.canVoidMilestone(node),
         "engRevokeDocument" :Evaluator.canRevokeEngDocument(node),
         "createMeetingMinutes": Evaluator.createMeetingMinutes(node),
         "startActionWorkflow": Evaluator.okForAction(node),
         "startInterfaceActionWorkflow": Evaluator.okForInterfaceAction(node),
         "startMilestoneWorkflow": Evaluator.canStartMilestoneWorkflow(node),
         "startVariationOrderWorkflow": Evaluator.canStartVariationOrderWorkflow(node),
         "startNcrWorkflow": Evaluator.canStartNcrWorkflow(node),
         "startIrlWorkflow": Evaluator.canStartIrlWorkflow(node),
         "startIasWorkflow": Evaluator.canStartIasWorkflow(node),
         "startEpbWorkflow": Evaluator.canStartEpbWorkflow(node),
         "startPunchlistWorkflow": Evaluator.canStartPunchlistWorkflow(node),
         "startIssuepunchlistWorkflow": Evaluator.canStartIssuepunchlistWorkflow(node),
         "startCivilPunchlistWorkflow": Evaluator.canStartCivilPunchlistWorkflow(node),
         "startRuiWorkflow": Evaluator.canStartRuiWorkflow(node),
         "startQAListWorkflow": Evaluator.canStartQAListWorkflow(node),
         "startCorrespondenceWorkflow": Evaluator.canStartCorrespondenceWorkflow(node),
         "startRiskWorkflow": Evaluator.canStartRiskWorkflow(node),
         "startHseActionWorkflow": Evaluator.canStartHseActionWorkflow(node),
         "startMccActionWorkflow": Evaluator.canStartMccActionWorkflow(node),
         "startActionWithProjectWorkflow": Evaluator.canStartActionWithProjectWorkflow(node),
         "startSiteQueryWorkflow": Evaluator.canStartSiteQueryWorkflow(node),
         "startHSEWorkflow": Evaluator.canStartHSEWorkflow(node),
         "startMCCPackagesWorkflow": Evaluator.canStartMCCPackagesWorkflow(node),
         "startCivilMeasuredQuantityWorkflow": Evaluator.canStartCivilMeasuredQuantityWorkflow(node),
         "startMCCCommissioningPackagesWorkflow": Evaluator.canStartMCCCommissioningPackagesWorkflow(node),
         "showWorkflowHistory": Evaluator.canShowWorkflowHistory(node),
         "showShareHistory": Evaluator.canShowShareHistory(node),
         "showEditors": Evaluator.canShowEditors(node),
         "startShareForCommentingWorkflow": Evaluator.canStartShareWorkflow(node),
         "startShareForInformationWorkflow": Evaluator.canStartShareWorkflow(node),
         "startRiskReducingMeasuresWorkflow": Evaluator.canStartRiskReducingMeasuresWorkflow(node),
         "startCivilTechnicalQueryWorkflow": Evaluator.canStartCivilTechnicalQueryWorkflow(node),
         "startSiteQueryNgWorkflow": Evaluator.canStartSiteQueryNgWorkflow(node),
         "generateMeetingPdf": Evaluator.canGenerateMeetingPdf(node),
         "startNewMQVersion": Evaluator.canStartNewMQVersion(node),
         "startIFCEngWorkflow" : Evaluator.hasUploadNewIFCDrawingPermission(node),
         "startCivilMilestoneWorkflow": Evaluator.canStartCivilMilestoneWorkflow(node),
         "startUnosysFeedbackWorkflow": Evaluator.canStartUnosysFeedbackWorkflow(node),
         "canDeleteWorkflow": Evaluator.canDeleteWorkflow(node),
         "startInterfaceAccommodationWorkflow": Evaluator.canStartInterfaceAccommodationWorkflow(node),
         "updateOrderAccommodationTask": Evaluator.canUpdateOrderAccommodationTask(node),
         "startCivilCorrespondenceWorkflow": Evaluator.canStartCivilCorrespondenceWorkflow(node),
         "startCivilActionWorkflow": Evaluator.canStartCivilActionWorkflow(node),
         "startCivilVariationOrderWorkflow": Evaluator.canStartCivilVariationOrderWorkflow(node),
         "lockItem":scriptHelper.canLock(node),
         "startMCCSafeWorkPermitWorkflow": Evaluator.canStartMCCSafeWorkPermitWorkflow(node),
         "revokeSafeWorkPermit": Evaluator.canRevokeMCCSafeWorkPermit(node),
         "completeSafeWorkPermit": Evaluator.canCompleteMCCSafeWorkPermit(node),
         "completeCivilMeasuredQuantity" : Evaluator.canCompleteCivilMeasuredQuantity(node),
         "incompleteCivilMeasuredQuantity" : Evaluator.canIncompleteCivilMeasuredQuantity(node),
         "civilMilestoneSubmitInvoice" : Evaluator.canCivilMilestoneSubmitInvoice(node),
      };
   },

   /**
    * Node Evaluator - main entrypoint
    */
   run: function Evaluator_run(node, fields)
   {
      var permissions = {},
         actionSet = "",
         actionLabels = {},
         createdBy = Common.getPerson(node.properties["cm:creator"]),
         modifiedBy = Common.getPerson(node.properties["cm:modifier"]),
         nodeData = {};

      /**
       * PERMISSIONS
       */

      // Use the form service to parse the required properties
      scriptObj = formService.getForm("node", node.nodeRef, fields, fields);

      // Make sure we can quickly look-up the Field Definition within the formData loop...
      var objDefinitions = {};
      for each (formDef in scriptObj.fieldDefinitions)
      {
         objDefinitions[formDef.dataKeyName] = formDef;
      }

      // Populate the data model
      var formData = scriptObj.formData.data;
      for (var k in formData)
      {
         var isAssoc = k.indexOf("assoc") == 0,
            value = formData[k].value,
            values,
            type = isAssoc ? objDefinitions[k].endpointType : objDefinitions[k].dataType,
            endpointMany = isAssoc ? objDefinitions[k].endpointMany : false,
            objData =
            {
               type: type
            };

         logger.log("Type: " + type + ", endpointMany: " + endpointMany);
         if (value instanceof java.util.Date)
         {
            objData.value = utils.toISO8601(value);
            objData.displayValue = objData.value;
            nodeData[k] = objData;
         }
         else if (endpointMany)
         {
            if (value.length() > 0)
            {
               values = value.split(",");
               nodeData[k] = [];
               for each (value in values)
               {
                  var objLoop =
                  {
                     type: objData.type,
                     value: value,
                     displayValue: value
                  };

                  if (Evaluator.decorateFieldData(objLoop, node))
                  {
                     nodeData[k].push(objLoop);
                  }
               }
            }
         }
         else
         {
            if (logger.isLoggingEnabled()) {
               logger.log("k: " + k)
            }
            if (k == "prop_cm_creator") {
               // This is a text field, resolve the person so we can display the full name instead of only the username.
               // The decorateFieldData will take care of the rest, together with extended-datagrid.js..
               var person = people.getPerson(value);
               objData.value = person.nodeRef.toString();
               objData.type = person.typeShort;
            }
            else {
               objData.value = value;
               objData.displayValue = isAssoc ? value : Evaluator.translateField(objDefinitions[k],value);

				if(asTextConfig[node.type]){ 											// if datalist is set
					if(asTextConfig[node.type][k]){ 									// if property in datalist is set
						var assocs = node.assocs[asTextConfig[node.type][k]];			// if real datalistitem have an assoc
	  					if(assocs != null){												// Check if its a person
	  						if(assocs.length > 0) {
		  						var typeAssoc = node.assocs[asTextConfig[node.type][k]][0];
		  						if(typeAssoc.type == '{http://www.alfresco.org/model/content/1.0}person'){
		       						objData.metadata = 'personAsText';
		  						}
	  						}
	  					}
	  				}
				}

            }

            if (Evaluator.decorateFieldData(objData, node))
            {
               nodeData[k] = objData;
            }
         }
      }

      return(
      {
         node: node,
         nodeData: nodeData,
         actionSet: actionSet,
         actionPermissions: permissions,
         createdBy: createdBy,
         modifiedBy: modifiedBy,
         tags: node.tags,
         actionLabels: actionLabels,
         isLocked: node.isLocked
      });
   },

  los:{ // Functions for logistic schedule nodes
    hasStatusClosed: function(node){
      // return (node.properties['{http://ns.lyse.no/model/1.0}losStatusProp'] === 'Closed');
      return (node.properties['lyse:losStatusProp'] === 'Closed');
    },

    isLos: function(node){
      // return (node.type == "{http://ns.lyse.no/model/datalist/1.0}logisticScheduleDatalist");
      return (node.typeShort == "lysedl:logisticScheduleDatalist");
    }
  }

};
