/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. Icf not, see <http://www.gnu.org/licenses/>.
 */
var isAdmin = people.isAdmin(person);
var Evaluator =
{
   /**
    * Cache for cm:person objects
    */
   PeopleObjectCache: {},

   /**
    * Gets / caches a person object
    *
    * @method getPersonObject
    * @param nodeRef {string} NodeRef of a cm:person object
    */
   getPersonObject: function Evaluator_getPersonObject(nodeRef)
   {
      if (nodeRef == null || nodeRef == "")
      {
         return null;
      }

      if (typeof Evaluator.PeopleObjectCache[nodeRef] == "undefined")
      {
         var person = search.findNode(nodeRef);
         Evaluator.PeopleObjectCache[nodeRef] =
         {
            userName: person.properties.userName,
            firstName: person.properties.firstName,
            lastName: person.properties.lastName,
            displayName: (person.properties.firstName + " " + person.properties.lastName).replace(/^\s+|\s+$/g, "")
         };
         if (person.assocs["cm:avatar"] != null)
         {
            Evaluator.PeopleObjectCache[nodeRef].avatar = person.assocs["cm:avatar"][0];
         }
      }
      return Evaluator.PeopleObjectCache[nodeRef];
   },

    /**
     * Cache for cm:authorityContainer objects
     */
    AuthorityContainerObjectCache: {},

    /**
     * Gets / caches an authorityContainer object
     *
     * @method getAuthorityContainerObject
     * @param nodeRef {string} NodeRef of a cm:authority object
     */
    getAuthorityContainerObject: function Evaluator_getAuthorityContainerObject(nodeRef)
    {
        if (nodeRef == null || nodeRef == "")
        {
            return null;
        }

        if (typeof Evaluator.AuthorityContainerObjectCache[nodeRef] == "undefined")
        {
            var authority = search.findNode(nodeRef);
            Evaluator.AuthorityContainerObjectCache[nodeRef] =
            {
                authorityName: authority.properties.authorityName,
                displayName: authority.properties.authorityDisplayName.replace(/^\s+|\s+$/g, "")
            };
        }
        return Evaluator.AuthorityContainerObjectCache[nodeRef];
    },

   /**
    * Cache for nodes that are subtypes of cm:cmobject
    */
   ContentObjectCache: {},

   /**
    * Gets / caches a content object
    *
    * @method getContentObject
    * @param nodeRef {string} NodeRef
    */
   getContentObject: function Evaluator_getContentObject(nodeRef)
   {
      if (nodeRef == null || nodeRef == "")
      {
         return null;
      }

      if (typeof Evaluator.ContentObjectCache[nodeRef] == "undefined")
      {
         var node = search.findNode(nodeRef);
         try
         {
            Evaluator.ContentObjectCache[nodeRef] = node;
         }
         catch(e)
         {
            // Possibly a stale indexed node
            return null;
         }
      }
      return Evaluator.ContentObjectCache[nodeRef];
   },
   
   /**
    * Only allow deletion of draft data list items.
    * 
    * @method hasDeletePermission
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} false to prevent this field being added to the output stream.
    */
   hasDeletePermission: function Evaluator_getDeletePermission(node)
   {

      if (node.type == "{http://ns.lyse.no/model/datalist/1.0}document" || node.type == "{http://ns.lyse.no/model/datalist/1.0}meetingMinutes")
      {
         return node.hasPermission("Delete");
      }else{
         return node.hasPermission("Delete") && Evaluator.isDraft(node);
      }

   },
   /**
    * If this is a contractors document list record, there should be
    * an uploadNewRevision-action. We solve this by using a new permission
    * type. No active workflow allowed.
    * 
    * @method hasUploadNewRevisionPermission
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a cdlList (ie show the upload new revision action).
    */
   hasUploadNewRevisionPermission: function Evaluator_hasUploadNewRevisionPermission(node)
   {
	  if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList" ||node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList" ) 
		  && node.hasPermission("Write") 
		  && node.properties["lysedl:workflowId"] == null
		  && node.properties["lyse:cdlReviewStatus"] != "void"){
		  return true;
	  }else{
		  return false;
	  }
   },
 
   /**
    * If this is a contractors document list record which is not in an ongoing review 
    * workflow, there should be an submitForReview-action. We solve this by using a new permission
    * type.
    * 
    * @method okForReview
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a cdlList and its not in an existing workflow.
    */
   okForReview: function Evaluator_okForReview(node)
   {
	  var shortName = node.getSiteShortName();
	  var sitePreset = siteService.getSite(shortName).sitePreset;
		  
	  return ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList" || node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList")
		  		&& node.hasPermission("Write") 
		  		&& siteHelper.isContractorUser(shortName)
		  		&& node.properties["lysedl:workflowId"] == null
		  		&& node.properties["lyse:cdlReviewStatus"] != "void");
   },
   
   okForAction: function Evaluator_okForAction(node)
   {
	  return (node.type == "{http://ns.lyse.no/model/datalist/1.0}issueList"
           && node.hasPermission("Write")
           && node.properties["lysedl:issueStatus"] == "draft"
           && node.properties["lysedl:workflowId"] == null);
   },

   okForInterfaceAction: function Evaluator_okForInterfaceAction(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}interfaceAction"
      && node.hasPermission("Write")
      && node.properties["lysedl:interfaceActionStatus"] == "draft"
      && node.properties["lysedl:workflowId"] == null);
   },

   /**
    * @return {boolean}
    */
   canShowWorkflowHistory: function Evaluator_canShowWorkflowHistory(node)
   {
      return (node.properties["lysedl:historicalWorkflowIds"] != null);
   },

   /**
    * @return {boolean}
    */
   canShowShareHistory: function Evaluator_canShowShareHistory(node)
   {
      return (node.properties["lyse:shareHistoryWorkflowIds"] != null);
   },

   /**
    * Check if a node has a draft status.
    *
    * @param node
    * @return {boolean}
    */
   isDraft: function Evaluator_isDraft(node)
   {
       return (node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList" && node.properties["lyse:cdlReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList" && node.properties["lyse:cdlReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}variationOrderRequestItem" && (node.properties["lysedl:vorStatus"] == "vor_draft" || node.properties["lysedl:vorStatus"] == "vo_draft"))
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}issueList" && node.properties["lysedl:issueStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}interfaceAction" && node.properties["lysedl:interfaceActionStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}milestoneItem" && node.properties["lysedl:milestoneStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}ncrList" && node.properties["lysedl:ncrStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}irlDataList" && node.properties["lyse:irlReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}iasDataList" && node.properties["lyse:iasReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}epbDataList" && node.properties["lyse:epbReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}punchList" && node.properties["lyse:punchReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}qaList" && node.properties["lyse:qaReviewStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}correspondenceItem" && node.properties["lysedl:correspondenceStatus"] == "draft")
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}contact") // in interface site, special handling.
           || (node.type == "{http://ns.lyse.no/model/datalist/1.0}sjaDataList"  && node.properties["lyse:sjaStatus"] == "new");
   },

   /**
    * Checks if a node can be edited by the current user.
    *
    * @param node
    * @constructor
    */
   canEdit: function Evaluator_canEdit(node)
   {
	   return lyseWorkflowService.canEdit(node);
   },

   canStartVariationOrderWorkflow: function Evaluator_canStartVariationOrderWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}variationOrderRequestItem" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },

   canStartMilestoneWorkflow: function Evaluator_canStartMilestoneWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}milestoneItem" && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },

   canStartNcrWorkflow: function Evaluator_canStartNcrWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}ncrList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartIrlWorkflow: function Evaluator_canStartIrlWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}irlDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartIasWorkflow: function Evaluator_canStartIasWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}iasDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartEpbWorkflow: function Evaluator_canStartEpbWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}epbDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartPunchlistWorkflow: function Evaluator_canStartPunchlistWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}punchList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartQAListWorkflow: function Evaluator_canStartQAListWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}qaList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },
   canStartCorrespondenceWorkflow: function Evaluator_canStartCorrespondenceWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}correspondenceItem" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },

   canStartRiskWorkflow: function Evaluator_canStartRiskWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}riskDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null);
   },
   canStartHSEWorkflow: function Evaluator_canStartHSEWorkflow(node)
   {
      return (node.type == "{http://ns.lyse.no/model/datalist/1.0}sjaDataList" && node.hasPermission("Write") && node.properties["lysedl:workflowId"] == null && Evaluator.isDraft(node));
   },

   /**
    * If this is the contractors document list there should be an multiple submitForReview-action. 
    * We solve this by using a new permission type.
    * 
    * @method okForMultipleReview
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a cdlList and its not in an existing workflow.
    */
   okForMultipleReview: function Evaluator_okForMultipleReview(node)
   {
	  if ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList" || node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList")){
		  return true;
	  }else{
		  return false;
	  }
   },

   /**
    * If this is the contractors document list there should be an multiple void-action. 
    * We solve this by using a new permission type.
    * 
    * @method canVoidCdlDocument
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is a cdlList and its not in an existing workflow.
    */
   canVoidCdlDocument: function Evaluator_canVoidCdlDocument(node)
   {
	  return ((node.type == "{http://ns.lyse.no/model/datalist/1.0}cdlDataList" ||node.type == "{http://ns.lyse.no/model/datalist/1.0}contractDataList")
		  && node.properties["lyse:cdlReviewStatus"] != "void"
          && node.hasPermission("Write")
          && node.properties["lysedl:workflowId"] == null);
   },

   /**
    * Only true for project sites, issueLists, irlLists, embLists, vo/vor:s and company users
    * 
    * @method createMeetingMinutes
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} true if this is the correct list, and the user has the authority
    */
   createMeetingMinutes: function Evaluator_createMeetingMinutes(node)
   {
	  if (node.type == "{http://ns.lyse.no/model/datalist/1.0}issueList" || 
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}iasDataList" ||
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}irlDataList" ||
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}interfaceAction" ||
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}epbDataList" || 
			  node.type == "{http://ns.lyse.no/model/datalist/1.0}variationOrderRequestItem"){
		  var shortName = node.getSiteShortName();
		  var sitePreset = siteService.getSite(shortName).sitePreset;
		  if (sitePreset == "lyse-project" || sitePreset == "contractor-project"){
             return (siteHelper.isCompanyUser(shortName) || siteHelper.isContractorUser(shortName));
		  }else if (sitePreset == "interface-site"){
			  return true;
		  }
	  }
	  return false;
   },

   canStartShareWorkflow: function Evaluator_canStartShareWorkflow(node) {
      return node.hasPermission("Read");
   },

   /**
    * Generate displayValue and any extra metadata for this field
    *
    * @method decorateFieldData
    * @param objData {object} Object literal containing this field's data
    * @param node {ScriptNode} The list item node for this field
    * @return {Boolean} false to prevent this field being added to the output stream.
    */
   decorateFieldData: function Evaluator_decorateFieldData(objData, node)
   {
      var value = objData.value,
         type = objData.type,
         obj;

      if (type == "cm:authority")
      {
         // cm:authority is the super type of both cm:person and cm:authorityContainer, now find the correct type and use that
         var authority = search.findNode(value);
         if (logger.isLoggingEnabled()) {
           logger.log("Authority: " + authority)
         }
         type = authority.typeShort;
         if (logger.isLoggingEnabled()) {
            logger.log("Authority type: " + type)
         }
      }

      if (type == "cm:person")
      {
         obj = Evaluator.getPersonObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.displayName;
         objData.metadata = obj.userName;
      }
      else if (type == "cm:authorityContainer")
      {
         obj = Evaluator.getAuthorityContainerObject(value);
         if (obj == null)
         {
             return false;
         }
         objData.displayValue = obj.displayName;
         objData.metadata = obj.authorityName;
      }
      else if (type == "category")
      {
    	 obj = Evaluator.getContentObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.name;
      }

      else if (type == "cm:folder")
      {
         obj = Evaluator.getContentObject(value);
         if (obj == null)
         {
            return false;
         }
         objData.displayValue = obj.displayPath.substring(companyhome.name.length() + 1);
         objData.metadata = "container";
      }
      else if (type.indexOf(":") > 0 && node.isSubType("cm:cmobject"))
      {
         obj = Evaluator.getContentObject(value);
         if (obj == null || !obj.hasPermission("Read"))
         {
            return false;
         }
         objData.displayValue = obj.properties["cm:name"];
         objData.theType = type;
         objData.metadata = obj.isContainer ? "container" : "document";
      }
      return true;
   },
   
   /**
    * Translates a List fieldDefinition
    *
    * @method translateField
    * @param objDef {FieldDefinition} objDef
    * @param value {String} default value
    */
   translateField: function Evaluator_translateField(objDef, value)
   {
      if (objDef == null || objDef == "")
      {
         return null;
      }
      if (objDef.constraints != null) 
      {
         for ( var i=0, len= objDef.constraints.size(); i<len; ++i )
         {          
             if ("LIST" == objDef.constraints.get(i).type) 
             {
                var allowedV = objDef.constraints.get(i).parameters.allowedValues;
                for (var j=0; j<allowedV.size(); ++j )
                {   
                    var allowedVasString = "" + allowedV.get(j);
                    var allValSplit = allowedVasString.split("|");
                    if (value == allValSplit[0]) {
                       return allValSplit[1];
                    }
                }
              }
          }
      }
      return value;
   },
   
   /**
    * Node Evaluator - main entrypoint
    */
   run: function Evaluator_run(node, fields)
   {
      var permissions = {},
         actionSet = "",
         actionLabels = {},
         createdBy = Common.getPerson(node.properties["cm:creator"]),
         modifiedBy = Common.getPerson(node.properties["cm:modifier"]),
         nodeData = {};

      /**
       * PERMISSIONS
       */
      logger.log("BEFORE PERMISSIONS CHECK");
      try {
      permissions =
      {
         "admin": isAdmin,
         "create": node.hasPermission("CreateChildren"),
         "edit": Evaluator.canEdit(node),
         "delete": Evaluator.hasDeletePermission(node),
         "cdlRevise": Evaluator.hasUploadNewRevisionPermission(node),
         "cdlSubmitForReview": Evaluator.okForReview(node),
         "cdlVoidDocument" :Evaluator.canVoidCdlDocument(node),
         "createMeetingMinutes": Evaluator.createMeetingMinutes(node),
         "startActionWorkflow": Evaluator.okForAction(node),
         "startInterfaceActionWorkflow": Evaluator.okForInterfaceAction(node),
         "startMilestoneWorkflow": Evaluator.canStartMilestoneWorkflow(node),
         "startVariationOrderWorkflow": Evaluator.canStartVariationOrderWorkflow(node),
         "startNcrWorkflow": Evaluator.canStartNcrWorkflow(node),
         "startIrlWorkflow": Evaluator.canStartIrlWorkflow(node),
         "startIasWorkflow": Evaluator.canStartIasWorkflow(node),
         "startEpbWorkflow": Evaluator.canStartEpbWorkflow(node),
         "startPunchlistWorkflow": Evaluator.canStartPunchlistWorkflow(node),
         "startQAListWorkflow": Evaluator.canStartQAListWorkflow(node),
         "startCorrespondenceWorkflow": Evaluator.canStartCorrespondenceWorkflow(node),
         "startRiskWorkflow": Evaluator.canStartRiskWorkflow(node),
         "startHSEWorkflow": Evaluator.canStartHSEWorkflow(node),
         "showWorkflowHistory": Evaluator.canShowWorkflowHistory(node),
         "showShareHistory": Evaluator.canShowShareHistory(node),
         "startShareForCommentingWorkflow": Evaluator.canStartShareWorkflow(node),
         "startShareForInformationWorkflow": Evaluator.canStartShareWorkflow(node)
      };

      }
      catch (ex) {
         logger.log(ex);
         throw ex;
      }
      logger.log("AFTER PERMISSIONS CHECK");

      // Use the form service to parse the required properties
      scriptObj = formService.getForm("node", node.nodeRef, fields, fields);

      // Make sure we can quickly look-up the Field Definition within the formData loop...
      var objDefinitions = {};
      for each (formDef in scriptObj.fieldDefinitions)
      {
         objDefinitions[formDef.dataKeyName] = formDef;
      }

      // Populate the data model
      var formData = scriptObj.formData.data;
      for (var k in formData)
      {
         var isAssoc = k.indexOf("assoc") == 0,
            value = formData[k].value,
            values,
            type = isAssoc ? objDefinitions[k].endpointType : objDefinitions[k].dataType,
            endpointMany = isAssoc ? objDefinitions[k].endpointMany : false,
            objData =
            {
               type: type
            };

         logger.log("Type: " + type + ", endpointMany: " + endpointMany);
         if (value instanceof java.util.Date)
         {
            objData.value = utils.toISO8601(value);
            objData.displayValue = objData.value;
            nodeData[k] = objData;
         }
         else if (endpointMany)
         {
            if (value.length() > 0)
            {
               values = value.split(",");
               nodeData[k] = [];
               for each (value in values)
               {
                  var objLoop =
                  {
                     type: objData.type,
                     value: value,
                     displayValue: value
                  };

                  if (Evaluator.decorateFieldData(objLoop, node))
                  {
                     nodeData[k].push(objLoop);
                  }
               }
            }
         }
         else
         {
            if (logger.isLoggingEnabled()) {
               logger.log("k: " + k)
            }
            if (k == "prop_cm_creator") {
               // This is a text field, resolve the person so we can display the full name instead of only the username.
               // The decorateFieldData will take care of the rest, together with extended-datagrid.js..
               var person = people.getPerson(value);
               objData.value = person.nodeRef.toString();
               objData.type = person.typeShort;
            }
            else {
               objData.value = value;
               objData.displayValue = isAssoc ? value : Evaluator.translateField(objDefinitions[k],value);
            }

            if (Evaluator.decorateFieldData(objData, node))
            {
               nodeData[k] = objData;
            }
         }
      }

      return(
      {
         node: node,
         nodeData: nodeData,
         actionSet: actionSet,
         actionPermissions: permissions,
         createdBy: createdBy,
         modifiedBy: modifiedBy,
         tags: node.tags,
         actionLabels: actionLabels,
         workflowTasks: lyseWorkflowService.getAssignedTasksForDatalist(node, true),
         responsibles: lyseWorkflowService.getResponsiblesForDatalistItem(node)
      });
   }
};
